#if defined (WIN32)
#include <windows.h>
#include <io.h>
#endif

#include <iostream>
#include <stdio.h>
#include <map>
#include <Producer/Utils.h>

#if defined( __linux )
#include <unistd.h>
#include <dlfcn.h>
#endif

namespace Producer {

#if defined (__linux )
static std::map<std::string, void *> libs;
#elif defined( WIN32 )
static std::map<std::string, HINSTANCE> libs;
#else
#endif


void (*getProcAddress( const std::string &libraryName, const char *procName ))(void)
{
    void (*sym)(void) = 0L;
#if defined ( __linux )
    void *handle = libs[libraryName];
    if( handle == 0L )
    {
        handle = libs[libraryName] = dlopen( libraryName.c_str(), RTLD_LAZY );
        if( handle == 0L )
        {
            std::cerr << "Producer::getProcAddress(" << libraryName << ", " << procName << "): Library Load Error : " <<  dlerror() << std::endl;
            return 0L;
        }
    }
    sym = (void (*)(void))dlsym(handle,(const char *)procName);
    return (void (*)(void))sym;
#elif defined( WIN32 )
    HINSTANCE handle = libs[libraryName];
    if( handle == 0L )
    {
        handle = libs[libraryName] = LoadLibrary( libraryName.c_str() );
        if( handle == 0L )
        {
            std::cerr << 
                "Producer::getProcAddress(" << libraryName << ", " << procName << "): Library Load Error : " <<  GetLastError() << "." << std::endl;
            return 0L;
        }
    }
    sym = (void (*)(void))GetProcAddress( handle, procName );
#endif
    return sym;
}

void (*getProcAddress(const GLubyte *procName))(void)
{
#if defined ( __linux )
    static void *handle = dlopen( (const char *)0L, RTLD_LAZY );
    return (void (*)(void))dlsym(handle,(const char *)procName);
#elif defined( WIN32 )
    return (void (*)(void))wglGetProcAddress( (LPCSTR)procName );
#else
    return 0L;
#endif
}

void (*getOpenGLProcAddress(const GLubyte *procName))(void)
{
    void (*symbol)(void) = getProcAddress( procName);

    // If not the base name, try an ARB extension
    if( symbol == NULL )
    {
        char arbName[128];
        sprintf(arbName, "%sARB", procName );
        symbol = getProcAddress( (const GLubyte *)arbName );
    }
    // If not an ARB extension  name, try an EXT extension
    if( symbol == NULL )
    {
        char extName[128];
        sprintf(extName, "%sEXT", procName );
        symbol = getProcAddress( (const GLubyte *)extName);
    }

    return symbol;
}

std::string getLibraryName( const std::string &base )
{
#if defined ( WIN32 )
    return base + ".dll";
#elif defined ( DARWIN )
    return base + ".dyl";
#else // All unices
    return "lib" + base + ".so";
#endif
}

bool fileExists(const std::string& filename)
{
#ifndef F_OK
#define F_OK 4
#endif
    return access( filename.c_str(), F_OK ) == 0;
}


std::string findLibrary( const std::string &libname )
{
    std::string path;
#if defined (WIN32)
#elif defined( DARWIN )
#else
    char *paths[] = {
        "/lib/",
        "/usr/lib/",
        "/usr/local/lib/"
    };

    char **p = paths;
    while( *p )
    {
        path = std::string(*p) + libname;
        if( fileExists( path ) )
            return path;
        p++;
    }
    path.clear();
#endif 
    return path;
}

}

